@@ -52,7 +52,10 @@ class BgpPlugin(service_base.ServicePluginBase,
         super(BgpPlugin, self).__init__()
         self.bgp_drscheduler = importutils.import_object(
             cfg.CONF.bgp_drscheduler_driver)
-        self._setup_rpc()
+        self.agent_notifiers[bgp_consts.AGENT_TYPE_BGP_ROUTING] = (
+            bgp_dr_rpc_agent_api.BgpDrAgentNotifyApi()
+        )
+        self._bgp_rpc = self.agent_notifiers[bgp_consts.AGENT_TYPE_BGP_ROUTING]
         self._register_callbacks()
         self.add_periodic_dragent_status_check()
 
@@ -64,17 +67,13 @@ class BgpPlugin(service_base.ServicePluginBase,
         return ("BGP dynamic routing service for announcement of next-hops "
                 "for project networks, floating IP's, and DVR host routes.")
 
-    def _setup_rpc(self):
+    def start_rpc_listeners(self):
         self.topic = bgp_consts.BGP_PLUGIN
         self.conn = n_rpc.Connection()
-        self.agent_notifiers[bgp_consts.AGENT_TYPE_BGP_ROUTING] = (
-            bgp_dr_rpc_agent_api.BgpDrAgentNotifyApi()
-        )
-        self._bgp_rpc = self.agent_notifiers[bgp_consts.AGENT_TYPE_BGP_ROUTING]
         self.endpoints = [bs_rpc.BgpSpeakerRpcCallback()]
         self.conn.create_consumer(self.topic, self.endpoints,
                                   fanout=False)
-        self.conn.consume_in_threads()
+        return self.conn.consume_in_threads()
 
     def _register_callbacks(self):
         registry.subscribe(self.floatingip_update_callback,
@@ -98,6 +97,9 @@ class BgpPlugin(service_base.ServicePluginBase,
         registry.subscribe(self.router_gateway_callback,
                            resources.ROUTER_GATEWAY,
                            events.AFTER_DELETE)
+        registry.subscribe(self.router_callback,
+                           resources.ROUTER,
+                           events.AFTER_UPDATE)
         registry.subscribe(self.port_callback,
                            resources.PORT,
                            events.AFTER_UPDATE)
@@ -284,7 +286,7 @@ class BgpPlugin(service_base.ServicePluginBase,
             return
 
         ctx = context.get_admin_context()
-        with ctx.session.begin(subtransactions=True):
+        with ctx.session.begin():
             speakers = self._bgp_speakers_for_gateway_network(ctx,
                                                               gw_network)
             next_hops = self._next_hops_from_gateway_ips(
@@ -303,35 +305,24 @@ class BgpPlugin(service_base.ServicePluginBase,
                                                     rl)
 
     def router_gateway_callback(self, resource, event, trigger, payload=None):
-        if event == events.AFTER_CREATE:
-            self._handle_router_gateway_after_create(payload)
-        if event == events.AFTER_DELETE:
-            gw_network = payload.metadata.get('network_id')
-            router_id = payload.resource_id
-            next_hops = self._next_hops_from_gateway_ips(
-                payload.metadata.get('gateway_ips'))
-            ctx = context.get_admin_context()
-            speakers = self._bgp_speakers_for_gateway_network(ctx, gw_network)
-            for speaker in speakers:
-                if speaker.ip_version in next_hops:
-                    next_hop = next_hops[speaker.ip_version]
-                    prefixes = self._tenant_prefixes_by_router(ctx,
-                                                               router_id,
-                                                               speaker.id)
-                    routes = self._route_list_from_prefixes_and_next_hop(
-                                                                     prefixes,
-                                                                     next_hop)
-                self._handle_router_interface_after_delete(gw_network, routes)
-
-    def _handle_router_gateway_after_create(self, payload):
-        ctx = context.get_admin_context()
         gw_network = payload.metadata.get('network_id')
         router_id = payload.resource_id
-        with ctx.session.begin(subtransactions=True):
+        gateway_ips = payload.metadata.get('gateway_ips')
+        if event == events.AFTER_CREATE:
+            self._handle_router_gateway_after_create(gw_network, router_id,
+                                                     gateway_ips)
+        if event == events.AFTER_DELETE:
+            self._handle_router_gateway_after_delete(gw_network, router_id,
+                                                     gateway_ips)
+
+    def _handle_router_gateway_after_create(self, gw_network, router_id,
+                                            gateway_ips):
+        ctx = context.get_admin_context()
+        with ctx.session.begin():
             speakers = self._bgp_speakers_for_gateway_network(ctx,
                                                               gw_network)
             next_hops = self._next_hops_from_gateway_ips(
-                payload.metadata.get('gateway_ips'))
+                gateway_ips)
 
             for speaker in speakers:
                 if speaker.ip_version in next_hops:
@@ -345,6 +336,22 @@ class BgpPlugin(service_base.ServicePluginBase,
                     self.start_route_advertisements(ctx, self._bgp_rpc,
                                                     speaker.id, routes)
 
+    def _handle_router_gateway_after_delete(self, gw_network, router_id,
+                                            gateway_ips):
+        next_hops = self._next_hops_from_gateway_ips(gateway_ips)
+        ctx = context.get_admin_context()
+        speakers = self._bgp_speakers_for_gateway_network(ctx, gw_network)
+        for speaker in speakers:
+            if speaker.ip_version in next_hops:
+                next_hop = next_hops[speaker.ip_version]
+                prefixes = self._tenant_prefixes_by_router(ctx,
+                                                           router_id,
+                                                           speaker.id)
+                routes = self._route_list_from_prefixes_and_next_hop(
+                                                                    prefixes,
+                                                                    next_hop)
+            self._handle_router_interface_after_delete(gw_network, routes)
+
     def _handle_router_interface_after_delete(self, gw_network, routes):
         if gw_network and routes:
             ctx = context.get_admin_context()
@@ -353,6 +360,32 @@ class BgpPlugin(service_base.ServicePluginBase,
                 self.stop_route_advertisements(ctx, self._bgp_rpc,
                                                speaker.id, routes)
 
+    def router_callback(self, resource, event, trigger, payload):
+        if event != events.AFTER_UPDATE:
+            return
+
+        original_router = payload.states[0]
+        updated_router = payload.latest_state
+        external_gateway_info = updated_router.get("external_gateway_info")
+        if external_gateway_info is None:
+            return
+
+        gateway_ips = [
+            info["ip_address"] for info in
+            external_gateway_info["external_fixed_ips"]]
+
+        if "admin_state_up" in original_router and \
+                original_router.get("admin_state_up") != \
+                updated_router.get("admin_state_up"):
+            router_id = updated_router["id"]
+            gw_network = external_gateway_info["network_id"]
+            if updated_router.get("admin_state_up"):
+                self._handle_router_gateway_after_create(gw_network, router_id,
+                                                         gateway_ips)
+            else:
+                self._handle_router_gateway_after_delete(gw_network, router_id,
+                                                         gateway_ips)
+
     def port_callback(self, resource, event, trigger, payload):
         if event != events.AFTER_UPDATE:
             return
@@ -369,7 +402,7 @@ class BgpPlugin(service_base.ServicePluginBase,
         # if host in the port binding has changed, update next-hops
         if original_host != updated_host and bool('compute:' in device_owner):
             ctx = context.get_admin_context()
-            with ctx.session.begin(subtransactions=True):
+            with ctx.session.begin():
                 ext_nets = self.get_external_networks_for_port(ctx,
                                                                updated_port)
                 for ext_net in ext_nets:
