@@ -134,35 +134,58 @@ class BgpEntityCreationMixin(object):
                                               ext_net_use_addr_scope=True,
                                               tenant_net_use_addr_scope=True):
         gw_ip_net = netaddr.IPNetwork(gw_prefix)
-        tenant_ip_net = netaddr.IPNetwork(tenant_prefix)
         ext_pool_args = {'tenant_id': tenant_id,
                          'name': 'bgp-pool'}
-        tenant_pool_args = ext_pool_args.copy()
 
         if address_scope and ext_net_use_addr_scope:
             ext_pool_args['address_scope_id'] = address_scope['id']
 
-        if address_scope and tenant_net_use_addr_scope:
-            tenant_pool_args['address_scope_id'] = address_scope['id']
-
-        with self.gw_network(external=True) as ext_net,\
-            self.network() as int_net,\
-            self.subnetpool([gw_prefix], **ext_pool_args) as ext_pool,\
-            self.subnetpool([tenant_prefix], **tenant_pool_args) as int_pool:
+        with self.gw_network(external=True) as ext_net, \
+                self.subnetpool([gw_prefix], **ext_pool_args) as ext_pool:
             ext_subnetpool_id = ext_pool['subnetpool']['id']
-            int_subnetpool_id = int_pool['subnetpool']['id']
             gw_net_id = ext_net['network']['id']
             with self.subnet(ext_net,
                              cidr=gw_prefix,
                              subnetpool_id=ext_subnetpool_id,
                              ip_version=gw_ip_net.version,
-                             as_admin=True),\
-                self.subnet(int_net,
+                             as_admin=True):
+                with self.router_with_tenant_networks(
+                    tenant_id,
+                    external_network_id=gw_net_id,
+                    tenant_prefix=tenant_prefix,
+                    address_scope=address_scope,
+                    distributed=distributed,
+                    ha=ha,
+                    tenant_net_use_addr_scope=tenant_net_use_addr_scope
+                ) as (router, int_net):
+                    yield router, ext_net, int_net
+
+    @contextlib.contextmanager
+    def router_with_tenant_networks(
+                                    self,
+                                    tenant_id,
+                                    external_network_id,
+                                    tenant_prefix='192.168.0.0/16',
+                                    address_scope=None,
+                                    distributed=False,
+                                    ha=False,
+                                    tenant_net_use_addr_scope=True):
+        tenant_ip_net = netaddr.IPNetwork(tenant_prefix)
+        tenant_pool_args = {'tenant_id': tenant_id,
+                            'name': 'bgp-pool'}
+
+        if address_scope and tenant_net_use_addr_scope:
+            tenant_pool_args['address_scope_id'] = address_scope['id']
+
+        with self.network() as int_net, \
+            self.subnetpool([tenant_prefix], **tenant_pool_args) as int_pool:
+            int_subnetpool_id = int_pool['subnetpool']['id']
+            with self.subnet(int_net,
                             cidr=tenant_prefix,
                             subnetpool_id=int_subnetpool_id,
                             ip_version=tenant_ip_net.version,
                             as_admin=True) as int_subnet:
-                ext_gw_info = {'network_id': gw_net_id}
+                ext_gw_info = {'network_id': external_network_id}
                 with self.router(external_gateway_info=ext_gw_info,
                                  distributed=distributed,
                                  ha=ha) as router:
@@ -172,7 +195,7 @@ class BgpEntityCreationMixin(object):
                     self.l3plugin.add_router_interface(self.context,
                                                        router_id,
                                                        router_interface_info)
-                    yield router, ext_net, int_net
+                    yield router, int_net
 
 
 class BgpTests(BgpEntityCreationMixin):
@@ -208,8 +231,8 @@ class BgpTests(BgpEntityCreationMixin):
 
     def test_add_duplicate_bgp_peer_ip(self):
         peer_ip = '192.168.1.10'
-        with self.bgp_peer(peer_ip=peer_ip) as peer1,\
-            self.bgp_peer(peer_ip=peer_ip) as peer2,\
+        with self.bgp_peer(peer_ip=peer_ip) as peer1, \
+            self.bgp_peer(peer_ip=peer_ip) as peer2, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=['8.0.0.0/8']) as sp:
 
@@ -233,11 +256,11 @@ class BgpTests(BgpEntityCreationMixin):
 
     def test_bgp_speaker_list(self):
         with self.subnetpool_with_address_scope(4,
-                                              prefixes=['8.0.0.0/8']) as sp1,\
+                                              prefixes=['8.0.0.0/8']) as sp1, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=['9.0.0.0/8']) as sp2:
             with self.bgp_speaker(sp1['ip_version'], 1234,
-                                  name='speaker1'),\
+                                  name='speaker1'), \
                 self.bgp_speaker(sp2['ip_version'], 4321,
                                  name='speaker2'):
                 speakers = self.bgp_plugin.get_bgp_speakers(self.context)
@@ -320,7 +343,7 @@ class BgpTests(BgpEntityCreationMixin):
                           'unreal-bgp-peer-id')
 
     def test_associate_bgp_peer(self):
-        with self.bgp_peer() as peer,\
+        with self.bgp_peer() as peer, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=['8.0.0.0/8']) as sp:
             with self.bgp_speaker(sp['ip_version'], 1234) as speaker:
@@ -333,7 +356,7 @@ class BgpTests(BgpEntityCreationMixin):
                 self.assertEqual(1, len(new_speaker['peers']))
 
     def test_remove_bgp_peer(self):
-        with self.bgp_peer() as peer,\
+        with self.bgp_peer() as peer, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=['8.0.0.0/8']) as sp:
             with self.bgp_speaker(sp['ip_version'], 1234,
@@ -346,7 +369,7 @@ class BgpTests(BgpEntityCreationMixin):
                 self.assertTrue(not new_speaker['peers'])
 
     def test_remove_unassociated_bgp_peer(self):
-        with self.bgp_peer() as peer,\
+        with self.bgp_peer() as peer, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=['8.0.0.0/8']) as sp:
             with self.bgp_speaker(sp['ip_version'], 1234) as speaker:
@@ -411,7 +434,7 @@ class BgpTests(BgpEntityCreationMixin):
     def test_add_gateway_network(self):
         with self.subnetpool_with_address_scope(4,
                                                 prefixes=['8.0.0.0/8']) as sp:
-            with self.bgp_speaker(sp['ip_version'], 1234) as speaker,\
+            with self.bgp_speaker(sp['ip_version'], 1234) as speaker, \
                 self.gw_network() as network:
                 network_id = network['network']['id']
                 self.bgp_plugin.add_gateway_network(self.context,
@@ -420,22 +443,22 @@ class BgpTests(BgpEntityCreationMixin):
                 new_speaker = self.bgp_plugin.get_bgp_speaker(self.context,
                                                               speaker['id'])
                 self.assertEqual(1, len(new_speaker['networks']))
-                self.assertTrue(network_id in new_speaker['networks'])
+                self.assertIn(network_id, new_speaker['networks'])
 
     def test_create_bgp_speaker_with_network(self):
-        with self.subnetpool_with_address_scope(4,
-                                                prefixes=['8.0.0.0/8']) as sp,\
+        with self.subnetpool_with_address_scope(
+                4, prefixes=['8.0.0.0/8']) as sp, \
             self.gw_network(name='test-net', tenant_id=_uuid(),
                             shared=True, as_admin=True) as network:
             network_id = network['network']['id']
             with self.bgp_speaker(sp['ip_version'], 1234,
                                   networks=[network_id]) as speaker:
                 self.assertEqual(1, len(speaker['networks']))
-                self.assertTrue(network_id in speaker['networks'])
+                self.assertIn(network_id, speaker['networks'])
 
     def test_remove_gateway_network(self):
-        with self.gw_network() as network1,\
-            self.gw_network() as network2,\
+        with self.gw_network() as network1, \
+            self.gw_network() as network2, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=['8.0.0.0/8']) as sp:
             network1_id = network1['network']['id']
@@ -473,8 +496,8 @@ class BgpTests(BgpEntityCreationMixin):
     def test_add_gateway_network_two_bgp_speakers_same_scope(self):
         with self.subnetpool_with_address_scope(4,
                                                 prefixes=['8.0.0.0/8']) as sp:
-            with self.bgp_speaker(sp['ip_version'], 1234) as speaker1,\
-                self.bgp_speaker(sp['ip_version'], 4321) as speaker2,\
+            with self.bgp_speaker(sp['ip_version'], 1234) as speaker1, \
+                self.bgp_speaker(sp['ip_version'], 4321) as speaker2, \
                 self.gw_network() as network:
                 network_id = network['network']['id']
                 self.bgp_plugin.add_gateway_network(self.context,
@@ -503,17 +526,17 @@ class BgpTests(BgpEntityCreationMixin):
         prefixes2 = ['9.0.0.0/8']
         prefixes3 = ['10.0.0.0/8']
         tenant_id = _uuid()
-        with self.bgp_speaker(4, 1234) as speaker,\
+        with self.bgp_speaker(4, 1234) as speaker, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=prefixes1,
-                                               tenant_id=tenant_id) as sp1,\
+                                               tenant_id=tenant_id) as sp1, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=prefixes2,
-                                               tenant_id=tenant_id) as sp2,\
+                                               tenant_id=tenant_id) as sp2, \
             self.subnetpool_with_address_scope(4,
                                                prefixes=prefixes3,
-                                               tenant_id=tenant_id) as sp3,\
-            self.gw_network() as network1, self.gw_network() as network2,\
+                                               tenant_id=tenant_id) as sp3, \
+            self.gw_network() as network1, self.gw_network() as network2, \
             self.gw_network() as network3:
             network1_id = network1['network']['id']
             network2_id = network2['network']['id']
@@ -554,8 +577,8 @@ class BgpTests(BgpEntityCreationMixin):
                                                                 self.context,
                                                                 speaker['id'])
             self.assertEqual(2, len(scopes))
-            self.assertTrue(sp1['address_scope_id'] in scopes)
-            self.assertTrue(sp2['address_scope_id'] in scopes)
+            self.assertIn(sp1['address_scope_id'], scopes)
+            self.assertIn(sp2['address_scope_id'], scopes)
 
     def test_get_routes_by_bgp_speaker_binding(self):
         gw_prefix = '172.16.10.0/24'
@@ -649,7 +672,7 @@ class BgpTests(BgpEntityCreationMixin):
                 return routes['advertised_routes']
 
     def test__tenant_prefixes_by_router_no_gateway_port(self):
-        with self.network() as net1, self.network() as net2,\
+        with self.network() as net1, self.network() as net2, \
             self.subnetpool_with_address_scope(6, tenant_id=_uuid(),
                                           prefixes=['2001:db8::/63']) as pool:
             subnetpool_id = pool['id']
@@ -657,12 +680,12 @@ class BgpTests(BgpEntityCreationMixin):
                              cidr=None,
                              subnetpool_id=subnetpool_id,
                              ip_version=6,
-                             as_admin=True) as ext_subnet,\
+                             as_admin=True) as ext_subnet, \
                 self.subnet(network=net2,
                             cidr=None,
                             subnetpool_id=subnetpool_id,
                             ip_version=6,
-                            as_admin=True) as int_subnet,\
+                            as_admin=True) as int_subnet, \
                 self.router() as router:
 
                 router_id = router['id']
@@ -723,10 +746,10 @@ class BgpTests(BgpEntityCreationMixin):
         binding_cidr = '2001:db8::/64'
         tenant_cidr = '2002:ab8::/64'
         with self.subnetpool_with_address_scope(6, tenant_id=_uuid(),
-                                       prefixes=[binding_cidr]) as ext_pool,\
+                                       prefixes=[binding_cidr]) as ext_pool, \
             self.subnetpool_with_address_scope(6, tenant_id=_uuid(),
-                                       prefixes=[tenant_cidr]) as int_pool,\
-            self.gw_network(external=True) as ext_net,\
+                                       prefixes=[tenant_cidr]) as int_pool, \
+            self.gw_network(external=True) as ext_net, \
             self.network() as int_net:
             gw_net_id = ext_net['network']['id']
             ext_pool_id = ext_pool['id']
@@ -735,12 +758,12 @@ class BgpTests(BgpEntityCreationMixin):
                              subnetpool_id=ext_pool_id,
                              network=ext_net,
                              ip_version=6,
-                             as_admin=True) as ext_subnet,\
+                             as_admin=True) as ext_subnet, \
                 self.subnet(cidr=None,
                             subnetpool_id=int_pool_id,
                             network=int_net,
                             ip_version=6,
-                            as_admin=True) as int_subnet,\
+                            as_admin=True) as int_subnet, \
                 self.router() as router:
                 router_id = router['id']
                 int_subnet_id = int_subnet['subnet']['id']
@@ -865,6 +888,141 @@ class BgpTests(BgpEntityCreationMixin):
                 self.assertTrue(tenant_prefix_found)
                 self.assertTrue(fip_prefix_found)
 
+    def test_get_routes_by_bgp_speaker_id_with_fip_router_disabled(self):
+        gw_prefix = '172.16.10.0/24'
+        tenant_prefix = '10.10.10.0/24'
+        tenant_id = _uuid()
+        scope_data = {'tenant_id': tenant_id, 'ip_version': 4,
+                      'shared': True, 'name': 'bgp-scope'}
+        scope = self.plugin.create_address_scope(
+                                                self.context,
+                                                {'address_scope': scope_data})
+        with self.router_with_external_and_tenant_networks(
+                                               tenant_id=tenant_id,
+                                               gw_prefix=gw_prefix,
+                                               tenant_prefix=tenant_prefix,
+                                               address_scope=scope) as res:
+            router, ext_net, int_net = res
+            self.l3plugin.update_router(
+                self.context, router['id'],
+                {'router': {'admin_state_up': False}})
+            gw_net_id = ext_net['network']['id']
+            tenant_net_id = int_net['network']['id']
+            fixed_port_data = {'port':
+                               {'name': 'test',
+                                'network_id': tenant_net_id,
+                                'tenant_id': tenant_id,
+                                'admin_state_up': True,
+                                'device_id': _uuid(),
+                                'device_owner': 'compute:nova',
+                                'mac_address': n_const.ATTR_NOT_SPECIFIED,
+                                'fixed_ips': n_const.ATTR_NOT_SPECIFIED}}
+            fixed_port = self.plugin.create_port(self.context,
+                                                 fixed_port_data)
+            fip_data = {'floatingip': {'floating_network_id': gw_net_id,
+                                       'tenant_id': tenant_id,
+                                       'port_id': fixed_port['id']}}
+            self.l3plugin.create_floatingip(self.context, fip_data)
+            with self.bgp_speaker(4, 1234, networks=[gw_net_id]) as speaker:
+                bgp_speaker_id = speaker['id']
+                routes = self.bgp_plugin.get_routes_by_bgp_speaker_id(
+                                                               self.context,
+                                                               bgp_speaker_id)
+                routes = list(routes)
+                print(routes)
+                self.assertEqual(0, len(routes))
+
+    def test_get_routes_by_bgp_speaker_id_with_fip_two_router_one_disabled(
+            self):
+        gw_prefix = '172.16.10.0/24'
+        tenant_prefix1 = '10.10.10.0/24'
+        tenant_prefix2 = '10.10.20.0/24'
+        tenant_id = _uuid()
+        scope_data = {'tenant_id': tenant_id, 'ip_version': 4,
+                      'shared': True, 'name': 'bgp-scope'}
+        scope = self.plugin.create_address_scope(
+                                                self.context,
+                                                {'address_scope': scope_data})
+        # network and router setup
+        with self.router_with_external_and_tenant_networks(
+                                               tenant_id=tenant_id,
+                                               gw_prefix=gw_prefix,
+                                               tenant_prefix=tenant_prefix1,
+                                               address_scope=scope) as res:
+            router1, ext_net, int_net1 = res
+            gw_net_id = ext_net['network']['id']
+            with self.router_with_tenant_networks(
+                tenant_id=tenant_id,
+                external_network_id=gw_net_id,
+                tenant_prefix=tenant_prefix2,
+                address_scope=scope
+            ) as (router2, int_net2):
+
+                # fip setup
+                tenant_net_id1 = int_net1['network']['id']
+                fixed_port_data1 = {'port':
+                                {'name': 'test',
+                                    'network_id': tenant_net_id1,
+                                    'tenant_id': tenant_id,
+                                    'admin_state_up': True,
+                                    'device_id': _uuid(),
+                                    'device_owner': 'compute:nova',
+                                    'mac_address': n_const.ATTR_NOT_SPECIFIED,
+                                    'fixed_ips': n_const.ATTR_NOT_SPECIFIED}}
+                fixed_port1 = self.plugin.create_port(self.context,
+                                                    fixed_port_data1)
+                fip_data1 = {'floatingip': {'floating_network_id': gw_net_id,
+                                        'tenant_id': tenant_id,
+                                        'port_id': fixed_port1['id']}}
+                fip1 = self.l3plugin.create_floatingip(self.context, fip_data1)
+
+                tenant_net_id2 = int_net2['network']['id']
+                fixed_port_data2 = {'port':
+                                {'name': 'test',
+                                    'network_id': tenant_net_id2,
+                                    'tenant_id': tenant_id,
+                                    'admin_state_up': True,
+                                    'device_id': _uuid(),
+                                    'device_owner': 'compute:nova',
+                                    'mac_address': n_const.ATTR_NOT_SPECIFIED,
+                                    'fixed_ips': n_const.ATTR_NOT_SPECIFIED}}
+                fixed_port2 = self.plugin.create_port(self.context,
+                                                    fixed_port_data2)
+                fip_data2 = {'floatingip': {'floating_network_id': gw_net_id,
+                                        'tenant_id': tenant_id,
+                                        'port_id': fixed_port2['id']}}
+                self.l3plugin.create_floatingip(self.context, fip_data2)
+
+                # disable router2
+                self.l3plugin.update_router(
+                    self.context, router2['id'],
+                    {'router': {'admin_state_up': False}})
+
+                # start bgp and validate
+                ext_gw_info1 = router1['external_gateway_info']
+                fip_prefix1 = fip1['floating_ip_address'] + '/32'
+
+                with self.bgp_speaker(4, 1234, networks=[gw_net_id]
+                                      ) as speaker:
+                    bgp_speaker_id = speaker['id']
+                    routes = self.bgp_plugin.get_routes_by_bgp_speaker_id(
+                                                                self.context,
+                                                                bgp_speaker_id)
+                    routes = list(routes)
+                    next_hop = ext_gw_info1[
+                        'external_fixed_ips'][0]['ip_address']
+                    self.assertEqual(2, len(routes))
+                    tenant_prefix_found = False
+                    fip_prefix_found = False
+                    for route in routes:
+                        self.assertEqual(next_hop, route['next_hop'])
+                        if route['destination'] == tenant_prefix1:
+                            tenant_prefix_found = True
+                        if route['destination'] == fip_prefix1:
+                            fip_prefix_found = True
+                    self.assertTrue(tenant_prefix_found)
+                    self.assertTrue(fip_prefix_found)
+
     def test_get_routes_by_bgp_speaker_id_with_fip_dvr(self):
         gw_prefix = '172.16.10.0/24'
         tenant_prefix = '10.10.10.0/24'
@@ -1104,8 +1262,8 @@ class BgpTests(BgpEntityCreationMixin):
                 self.assertTrue(fip_prefix_found)
 
     def test__bgp_speakers_for_gateway_network_by_ip_version(self):
-        with self.gw_network(external=True) as ext_net,\
-            self.bgp_speaker(6, 1234) as s1,\
+        with self.gw_network(external=True) as ext_net, \
+            self.bgp_speaker(6, 1234) as s1, \
             self.bgp_speaker(6, 4321) as s2:
             gw_net_id = ext_net['network']['id']
             self.bgp_plugin.add_gateway_network(self.context,
@@ -1121,8 +1279,8 @@ class BgpTests(BgpEntityCreationMixin):
             self.assertEqual(2, len(speakers))
 
     def test__bgp_speakers_for_gateway_network_by_ip_version_no_binding(self):
-        with self.gw_network(external=True) as ext_net,\
-            self.bgp_speaker(6, 1234),\
+        with self.gw_network(external=True) as ext_net, \
+            self.bgp_speaker(6, 1234), \
             self.bgp_speaker(6, 4321):
             gw_net_id = ext_net['network']['id']
             speakers = self.bgp_plugin._bgp_speakers_for_gw_network_by_family(
